package nori

import (
	"fmt"
	"image"
	"image/draw"

	"git.tuxpa.in/a/zlog/log"
	"gitlab.com/gfxlabs/gfximg/apng"
)

func (n *Nori) ShowAnimation() *apng.APNG {
	g := n.Gawi
	a := &apng.APNG{
		Frames: make([]apng.Frame, 0, len(g.Images)),
	}
	for i := 0; i < len(g.Images); i++ {
		fr := apng.Frame{
			Image:            g.Images[i].Img,
			DelayNumerator:   uint16(g.Images[i].Delay),
			DelayDenominator: 1000,
		}
		a.Frames = append(a.Frames, fr)
	}
	return a
}

func (n *Nori) ExportAnimation() (*apng.APNG, error) {
	g := n.Gawi
	a := &apng.APNG{
		Frames: make([]apng.Frame, 0, len(g.Images)),
	}
	for i := 0; i < len(n.Animations); i++ {
		for _, v := range n.Animations[i].Frames {
			canvasRect := image.Rect(0, 0, 0, 0)
			for _, plane := range v.Planes {
				if int(plane.BitmapId) >= len(n.Gawi.Images) {
					log.Printf("could not find bitmap %d, only have %d", plane.BitmapId, len(n.Gawi.Images))
				}
				bitmap := n.Gawi.Images[plane.BitmapId]
				canvasRect = canvasRect.Union(bitmap.Img.Bounds())
			}
			img := image.NewRGBA64(canvasRect)
			for _, plane := range v.Planes {
				bitmap := n.Gawi.Images[plane.BitmapId]
				transparent := false
				//flipx
				if plane.RenderFlag&1 != 0 {
				}
				//flipy
				if plane.RenderFlag&2 != 0 {
				}
				// is transparent
				if plane.RenderFlag&0x20 != 0 {
					transparent = true
				}
				_ = transparent
				switch plane.Blend {
				case BlendMode_Alpha:
				case BlendMode_Mul, BlendMode_Mul7:
				case BlendMode_Add, BlendMode_Add8:
				case BlendMode_InvertMul, BlendMode_InvertMul5:
				case BlendMode_None:
					break
				default:
					return nil, fmt.Errorf("unknown blend mode: %d", plane.Blend)
				}
				draw.Draw(img,
					img.Rect,
					bitmap.Img,
					image.Point{X: int(plane.PlaneX),
						Y: int(plane.PlaneY)}, draw.Over)
			}
			fr := apng.Frame{
				Image:            g.Images[i].Img,
				DelayNumerator:   uint16(g.Images[i].Delay),
				DelayDenominator: 1000,
			}
			a.Frames = append(a.Frames, fr)
		}
	}
	return a, nil
}
