package nori

import (
	"bytes"
	"os"
	"testing"

	"git.tuxpa.in/a/zlog/log"
	"gitlab.com/gfxlabs/gfximg/apng"
)

func TestParseFile1(t *testing.T) {
	rd, err := os.OpenFile("./nori_test/test1.nri", os.O_RDONLY, 0666)
	if err != nil {
		t.Errorf("open file: %s", err)
	}
	nori, err := NewReader(rd).Decode()
	if err != nil {
		t.Errorf("decode: %s", err)
	}
	t.Logf("\n nori: %+v", nori)
	t.Logf("\n gawi: %+v", nori.Gawi)
	a, err := nori.ExportAnimation()
	if err != nil {
		t.Errorf("export: %s", err)
	}
	writeApng(a, "./nori_test/test1.apng")
}

func TestParseFile2(t *testing.T) {
	rd, err := os.OpenFile("./nori_test/test2.nri", os.O_RDONLY, 0666)
	if err != nil {
		t.Errorf("open file: %s", err)
	}
	nori, err := NewReader(rd).Decode()
	if err != nil {
		t.Errorf("decode: %s", err)
	}
	t.Logf("\n nori: %+v\n gawi: %+v", nori, nori.Gawi)
	a, err := nori.ExportAnimation()
	if err != nil {
		t.Errorf("export: %s", err)
	}

	writeApng(a, "./nori_test/test2.apng")
}

func TestParsePalette(t *testing.T) {
	rd, err := os.OpenFile("./nori_test/palette.nri", os.O_RDONLY, 0666)
	if err != nil {
		t.Errorf("open file: %s", err)
	}
	nori, err := NewReader(rd).Decode()
	if err != nil {
		t.Errorf("decode: %s", err)
	}
	t.Logf("\n nori: %+v\n gawi: %+v\n palette: %+v\n", nori, nori.Gawi, nori.Gawi.Palette)
	a, err := nori.ExportAnimation()
	if err != nil {
		t.Errorf("export: %s", err)
	}
	writeApng(a, "./nori_test/palette.apng")
}

func writeApng(a *apng.APNG, fp string) {
	out := new(bytes.Buffer)
	err := apng.Encode(out, *a)
	if err != nil {
		log.Errorf("fail to encode apng %s", err)
	}
	os.WriteFile(fp, out.Bytes(), 0740)
}
